/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.novabpm.identity.impl;

import java.security.Permission;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Properties;
import java.util.Set;

import org.hibernate.Session;
import org.jbpm.env.Environment;
import org.jbpm.env.EnvironmentFactory;
import org.jbpm.id.db.IdentitySession;
import org.ow2.novabpm.identity.CommitException;
import org.ow2.novabpm.identity.GroupNotFoundException;
import org.ow2.novabpm.identity.GroupOp;
import org.ow2.novabpm.identity.IdentityServiceOp;
import org.ow2.novabpm.identity.Membership;
import org.ow2.novabpm.identity.UserNotFoundException;
import org.ow2.novabpm.identity.UserOp;

/**
 * Bridge between the PVM Identity module and the Bonita Identity module.
 * 
 * @author Pierre Vigneras
 * @date Jan 23, 2008
 */
public class PVMIdentityService implements IdentityServiceOp{
	private Collection<Object> modified = Collections.synchronizedList(new ArrayList<Object>());
	private final EnvironmentFactory factory;
	
	public PVMIdentityService(final EnvironmentFactory factory) {
		this.factory = factory;
	}
	
	private IdentitySession newSession() {
		final Environment environment = factory.openEnvironment();
		final Session session = environment.get(Session.class);
		if (session == null) {
			throw new RuntimeException("Can't find a session inside environment: " + environment);
		}
		IdentitySession idSession = new IdentitySession();
		idSession.setSession(session);
		return idSession;
	}
	
	/* (non-Javadoc)
	 * @see org.ow2.novabpm.identity.IdentityServiceOp#commit()
	 */
	public void commit() throws CommitException {
		final IdentitySession idSession = newSession();
		for (Object o : modified) {
			idSession.save(o);
		}
		idSession.getSession().close();
	}

	/* (non-Javadoc)
	 * @see org.ow2.novabpm.identity.IdentityServiceOp#createGroup()
	 */
	public String createGroup() {
		throw new RuntimeException("createGroup() is not implemented!");
	}

	/* (non-Javadoc)
	 * @see org.ow2.novabpm.identity.IdentityServiceOp#createGroup(org.ow2.novabpm.identity.GroupOp)
	 */
	public String createGroup(GroupOp parent) {
		throw new RuntimeException("createGroup(GroupOp parent) is not implemented!");
	}

	/* (non-Javadoc)
	 * @see org.ow2.novabpm.identity.IdentityServiceOp#createUser()
	 */
	public String createUser() {
		throw new RuntimeException("createUser() is not implemented!");
	}

	/* (non-Javadoc)
	 * @see org.ow2.novabpm.identity.IdentityServiceOp#deleteGroup(java.lang.String)
	 */
	public void deleteGroup(String id) throws GroupNotFoundException {
		throw new RuntimeException("deleteGroup(String) is not implemented!");
		
	}

	/* (non-Javadoc)
	 * @see org.ow2.novabpm.identity.IdentityServiceOp#deleteUser(java.lang.String)
	 */
	public void deleteUser(String id) throws UserNotFoundException {
		throw new RuntimeException("deleteUser(String) is not implemented!");
	}

	/* (non-Javadoc)
	 * @see org.ow2.novabpm.identity.IdentityServiceOp#getAllGroups()
	 */
	public Collection<GroupOp> getAllGroups() {
		final IdentitySession idSession = newSession();
		final List<org.jbpm.id.Group> groups = idSession.findAllGroups();
		final Collection<GroupOp> result = new ArrayList<GroupOp>();
		for (org.jbpm.id.Group group : groups) {
			result.add(new PVMGroup(group));
		}
		return result;
	}

	/* (non-Javadoc)
	 * @see org.ow2.novabpm.identity.IdentityServiceOp#getAllUsers()
	 */
	public Collection<UserOp> getAllUsers() {
		final IdentitySession idSession = newSession();
		final List<org.jbpm.id.User> users = idSession.findAllUsers();
		final Collection<UserOp> result = new ArrayList<UserOp>();
		for (org.jbpm.id.User user : users) {
			result.add(new PVMUser(user));
		}
		return result;
	}

	/* (non-Javadoc)
	 * @see org.ow2.novabpm.identity.IdentityServiceOp#getGroup(java.lang.String)
	 */
	public GroupOp getGroup(String id) throws GroupNotFoundException {
		final IdentitySession idSession = newSession();
		return new PVMGroup(idSession.findGroupById(id));
	}

	/* (non-Javadoc)
	 * @see org.ow2.novabpm.identity.IdentityServiceOp#getUser(java.lang.String)
	 */
	public UserOp getUser(String id) throws UserNotFoundException {
		final IdentitySession idSession = newSession();
		return new PVMUser(idSession.findUserById(id));
	}

	/* (non-Javadoc)
	 * @see org.ow2.novabpm.identity.IdentityServiceOp#modifyGroup(org.ow2.novabpm.identity.GroupOp)
	 */
	public boolean modifyGroup(GroupOp groupOp) throws GroupNotFoundException {
		final PVMGroup pvmGroup = (PVMGroup) groupOp;
		modified.add(pvmGroup.group);
		return true;
	}

	/* (non-Javadoc)
	 * @see org.ow2.novabpm.identity.IdentityServiceOp#modifyUser(org.ow2.novabpm.identity.UserOp)
	 */
	public boolean modifyUser(UserOp userOp) throws UserNotFoundException {
		final PVMUser pvmUser = (PVMUser) userOp;
		modified.add(pvmUser.user);
		return true;
	}

	/* (non-Javadoc)
	 * @see org.ow2.novabpm.identity.IdentityServiceOp#setMembership(org.ow2.novabpm.identity.UserOp, org.ow2.novabpm.identity.GroupOp)
	 */
	public Membership setMembership(UserOp user, GroupOp group)
			throws UserNotFoundException, GroupNotFoundException {
		final PVMUser pvmUser = (PVMUser) user;
		final PVMGroup pvmGroup = (PVMGroup) group;
		final IdentitySession idSession = newSession();
		org.jbpm.id.Membership membership = new org.jbpm.id.Membership(pvmUser.user, pvmGroup.group);
		pvmUser.user.getMemberships().add(membership);
		pvmGroup.group.getMemberships().add(membership);
		modified.add(pvmUser.user);
		modified.add(pvmGroup.group);
		return new PVMMembership(membership);
	}
	
	
	
	/**
	 * @author Pierre Vigneras
	 * @date Jan 30, 2008
	 */
	static class PVMMembership implements Membership {
		final org.jbpm.id.Membership membership;
		
		PVMMembership(org.jbpm.id.Membership membership) {
			this.membership = membership;
		}
		
		/* (non-Javadoc)
		 * @see org.ow2.novabpm.identity.Membership#getGroup()
		 */
		public GroupOp getGroup() {
			return new PVMGroup(membership.getGroup());
		}

		/* (non-Javadoc)
		 * @see org.ow2.novabpm.identity.Membership#getPermission()
		 */
		public Set<Permission> getPermission() {
			return membership.getPermissions();
		}

		/* (non-Javadoc)
		 * @see org.ow2.novabpm.identity.Membership#getProperties()
		 */
		public Properties getProperties() {
			return membership.getProperties();
		}

		/* (non-Javadoc)
		 * @see org.ow2.novabpm.identity.Membership#getRole()
		 */
		public String getRole() {
			return membership.getRole();
		}

		/* (non-Javadoc)
		 * @see org.ow2.novabpm.identity.Membership#getUser()
		 */
		public UserOp getUser() {
			return new PVMUser(membership.getUser());
		}

		/* (non-Javadoc)
		 * @see org.ow2.novabpm.identity.Membership#setPermissions(java.util.Set)
		 */
		public void setPermissions(Set<Permission> newPermissions) {
			membership.setPermissions(newPermissions);
		}

		/* (non-Javadoc)
		 * @see org.ow2.novabpm.identity.Membership#setProperties(java.util.Set)
		 */
		public void setProperties(Properties newProperties) {
			membership.setProperties(newProperties);
		}

		/* (non-Javadoc)
		 * @see org.ow2.novabpm.identity.Membership#setRole(javax.management.relation.Role)
		 */
		public void setRole(String newRole) {
			membership.setRole(newRole);
		}

	}

	/**
	 * @author Pierre Vigneras
	 * @date Jan 23, 2008
	 */
	static class PVMGroup implements GroupOp {
		final org.jbpm.id.Group group;
		
		PVMGroup(final org.jbpm.id.Group group) {
			this.group = group;
		}
		
		/* (non-Javadoc)
		 * @see org.ow2.novabpm.identity.GroupOp#getRoot()
		 */
		public GroupOp getRoot() {
			org.jbpm.id.Group candidate = group;
			while (true) {
				org.jbpm.id.Group parent = candidate.getParent();
				if (parent == null) {
					return new PVMGroup(candidate);
				}
				candidate = parent;
			}
		}

		/* (non-Javadoc)
		 * @see org.ow2.novabpm.identity.GroupOp#hasMember(org.ow2.novabpm.identity.UserOp)
		 */
		public boolean hasMember(UserOp userOp) {
			final PVMUser user = (PVMUser) userOp;
			final Set<org.jbpm.id.Membership> memberships = this.group.getMemberships();
			for (org.jbpm.id.Membership membership: memberships) {
				if (membership.getUser().equals(user)) {
					return true;
				}
			}
			return false;
		}

		/* (non-Javadoc)
		 * @see org.ow2.novabpm.identity.GroupOp#getChildren()
		 */
		public Set<GroupOp> getChildren() {
			final Set<GroupOp> result = new HashSet<GroupOp>();
			final Set<org.jbpm.id.Group> children = group.getChildren();
			for (org.jbpm.id.Group child: children) {
				result.add(new PVMGroup(group));
			}
			return result;
		}

		/* (non-Javadoc)
		 * @see org.ow2.novabpm.identity.GroupOp#getId()
		 */
		public String getId() {
			return group.getId();
		}

		/* (non-Javadoc)
		 * @see org.ow2.novabpm.identity.GroupOp#getMemberships()
		 */
		public Set<Membership> getMemberships() {
			final Set<Membership> result = new HashSet<Membership>();
			final Set<org.jbpm.id.Membership> memberships = this.group.getMemberships(); 
			for (org.jbpm.id.Membership membership: memberships) {
				result.add(new PVMMembership(membership));
			}
			return result;
		}

		/* (non-Javadoc)
		 * @see org.ow2.novabpm.identity.GroupOp#getParent()
		 */
		public GroupOp getParent() {
			final org.jbpm.id.Group parent = group.getParent();
			return new PVMGroup(parent);
		}

		/* (non-Javadoc)
		 * @see org.ow2.novabpm.identity.GroupOp#getPermissions()
		 */
		public Set<Permission> getPermissions() {
			return group.getPermissions();
		}

		/* (non-Javadoc)
		 * @see org.ow2.novabpm.identity.GroupOp#getProperties()
		 */
		public Properties getProperties() {
			final Properties properties = group.getProperties();
			properties.setProperty("Name", group.getName());
			properties.setProperty("Type", group.getType());
			return properties;
		}

		/* (non-Javadoc)
		 * @see org.ow2.novabpm.identity.GroupOp#setPermissions(java.util.Set)
		 */
		public void setPermissions(Set<Permission> permissions) {
			group.setPermissions(permissions);
		}

		/* (non-Javadoc)
		 * @see org.ow2.novabpm.identity.GroupOp#setProperties(java.util.Properties)
		 */
		public void setProperties(Properties newProperties) {
			final String name = (String) newProperties.remove("Name");
			final String type = (String) newProperties.remove("Type");
			group.setName(name == null ? group.getName() : name);
			group.setType(type == null ? group.getType() : type);
			group.setProperties(newProperties);
		}

	}

	/**
	 * @author Pierre Vigneras
	 * @date Jan 23, 2008
	 */
	static class PVMUser implements UserOp {
		final org.jbpm.id.User user;
		
		PVMUser(final org.jbpm.id.User user) {
			this.user = user;
		}
		
		/* (non-Javadoc)
		 * @see org.ow2.novabpm.identity.UserOp#getMemberships()
		 */
		public Set<Membership> getMemberships() {
			Set<Membership> result = new HashSet<Membership>();
			Set<org.jbpm.id.Membership> set = user.getMemberships();
			for (org.jbpm.id.Membership m: set) {
				result.add(new PVMMembership(m));
			}
			return result;
		}

		/* (non-Javadoc)
		 * @see org.ow2.novabpm.identity.UserOp#isMember(org.ow2.novabpm.identity.GroupOp)
		 */
		public boolean isMember(GroupOp groupOp) {
			final PVMGroup group = (PVMGroup) groupOp;
			final Set<org.jbpm.id.Membership> memberships = group.group.getMemberships();
			for (org.jbpm.id.Membership membership: memberships) {
				if (membership.getUser().equals(this.user)) {
					return true;
				}
			}
			return false;
		}

		/* (non-Javadoc)
		 * @see org.ow2.novabpm.identity.UserOp#getId()
		 */
		public String getId() {
			return user.getId();
		}

		/* (non-Javadoc)
		 * @see org.ow2.novabpm.identity.UserOp#getPermissions()
		 */
		public Set<Permission> getPermissions() {
			return user.getPermissions();
		}

		/* (non-Javadoc)
		 * @see org.ow2.novabpm.identity.UserOp#getProperties()
		 */
		public Properties getProperties() {
			final Properties properties = user.getProperties();
			properties.put("Address", user.getAddress());
			properties.put("Email", user.getEmail());
			properties.put("Name", user.getName());
			properties.put("Notes", user.getNotes());
			properties.put("Phone", user.getPhone());
			return properties;
		}

		/* (non-Javadoc)
		 * @see org.ow2.novabpm.identity.UserOp#setPermissions(java.util.Set)
		 */
		public void setPermissions(Set<Permission> permissions) {
			user.setPermissions(permissions);
		}

		/* (non-Javadoc)
		 * @see org.ow2.novabpm.identity.UserOp#setProperties(java.util.Properties)
		 */
		public void setProperties(Properties newProperties) {
			final String address = (String) newProperties.remove("Address");
			final String email = (String) newProperties.remove("Email");
			final String name = (String) newProperties.remove("Name");
			final String notes = (String) newProperties.remove("Notes");
			final String phone = (String) newProperties.remove("Phone");
			user.setAddress(address == null ? user.getAddress() : address);
			user.setEmail(email == null ? user.getEmail() : email);
			user.setName(name == null ? user.getName() : name);
			user.setNotes(notes == null ? user.getNotes() : notes);
			user.setPhone(phone == null ? user.getPhone() : phone);
			user.setProperties(newProperties);
		}
	}

}
